@@ -25,9 +25,6 @@ pyint_as_int(PyObject * pyint, int *pint)
 
 /* Interned strings to speed GetAttr etc. */
 
-static PyObject *str_trace;
-static PyObject *str_file_tracer;
-static PyObject *str__coverage_enabled;
 static PyObject *str__coverage_plugin;
 static PyObject *str__coverage_plugin_name;
 static PyObject *str_dynamic_source_filename;
@@ -44,9 +41,6 @@ CTracer_intern_strings(void)
         goto error;                             \
     }
 
-    INTERN_STRING(str_trace, "trace")
-    INTERN_STRING(str_file_tracer, "file_tracer")
-    INTERN_STRING(str__coverage_enabled, "_coverage_enabled")
     INTERN_STRING(str__coverage_plugin, "_coverage_plugin")
     INTERN_STRING(str__coverage_plugin_name, "_coverage_plugin_name")
     INTERN_STRING(str_dynamic_source_filename, "dynamic_source_filename")
@@ -102,6 +96,8 @@ CTracer_dealloc(CTracer *self)
     Py_XDECREF(self->should_trace_cache);
     Py_XDECREF(self->should_start_context);
     Py_XDECREF(self->switch_context);
+    Py_XDECREF(self->lock_data);
+    Py_XDECREF(self->unlock_data);
     Py_XDECREF(self->context);
     Py_XDECREF(self->disable_plugin);
 
@@ -292,6 +288,17 @@ error:
     return ret;
 }
 
+// Thanks for the idea, memray!
+static inline PyCodeObject*
+MyFrame_BorrowCode(PyFrameObject* frame)
+{
+    // Return a borrowed reference.
+    PyCodeObject* pCode = PyFrame_GetCode(frame);
+    assert(Py_REFCNT(pCode) >= 2);
+    Py_DECREF(pCode);
+    return pCode;
+}
+
 /*
  * Parts of the trace function.
  */
@@ -307,7 +314,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
     PyObject * plugin = NULL;
     PyObject * plugin_name = NULL;
     PyObject * next_tracename = NULL;
-#ifdef RESUME
+#ifdef RESUME   // >=3.11
     PyObject * pCode = NULL;
 #endif
 
@@ -363,7 +370,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
     }
 
     /* Check if we should trace this line. */
-    filename = MyFrame_GetCode(frame)->co_filename;
+    filename = MyFrame_BorrowCode(frame)->co_filename;
     disposition = PyDict_GetItem(self->should_trace_cache, filename);
     if (disposition == NULL) {
         if (PyErr_Occurred()) {
@@ -476,26 +483,39 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
     }
 
     if (tracename != Py_None) {
-        PyObject * file_data = PyDict_GetItem(self->data, tracename);
+        PyObject * file_data;
+        BOOL had_error = FALSE;
+        PyObject * res;
+
+        res = PyObject_CallFunctionObjArgs(self->lock_data, NULL);
+        if (res == NULL) {
+            goto error;
+        }
+
+        file_data = PyDict_GetItem(self->data, tracename);
 
         if (file_data == NULL) {
             if (PyErr_Occurred()) {
-                goto error;
+                had_error = TRUE;
+                goto unlock;
             }
             file_data = PySet_New(NULL);
             if (file_data == NULL) {
-                goto error;
+                had_error = TRUE;
+                goto unlock;
             }
             ret2 = PyDict_SetItem(self->data, tracename, file_data);
             if (ret2 < 0) {
-                goto error;
+                had_error = TRUE;
+                goto unlock;
             }
 
             /* If the disposition mentions a plugin, record that. */
             if (file_tracer != Py_None) {
                 ret2 = PyDict_SetItem(self->file_tracers, tracename, plugin_name);
                 if (ret2 < 0) {
-                    goto error;
+                    had_error = TRUE;
+                    goto unlock;
                 }
             }
         }
@@ -504,6 +524,17 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
             Py_INCREF(file_data);
         }
 
+        unlock:
+
+        res = PyObject_CallFunctionObjArgs(self->unlock_data, NULL);
+        if (res == NULL) {
+            goto error;
+        }
+
+        if (had_error) {
+            goto error;
+        }
+
         Py_XDECREF(self->pcur_entry->file_data);
         self->pcur_entry->file_data = file_data;
         self->pcur_entry->file_tracer = file_tracer;
@@ -514,15 +545,13 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
         Py_XDECREF(self->pcur_entry->file_data);
         self->pcur_entry->file_data = NULL;
         self->pcur_entry->file_tracer = Py_None;
-        frame->f_trace_lines = 0;
         SHOWLOG(PyFrame_GetLineNumber(frame), filename, "skipped");
     }
 
     self->pcur_entry->disposition = disposition;
 
     /* Make the frame right in case settrace(gettrace()) happens. */
-    Py_INCREF(self);
-    Py_XSETREF(frame->f_trace, (PyObject*)self);
+    MyFrame_SetTrace(frame, self);
 
     /* A call event is really a "start frame" event, and can happen for
      * re-entering a generator also.  How we tell the difference depends on
@@ -535,7 +564,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
      * The current opcode is guaranteed to be RESUME. The argument
      * determines what kind of resume it is.
      */
-    pCode = MyCode_GetCode(MyFrame_GetCode(frame));
+    pCode = MyCode_GetCode(MyFrame_BorrowCode(frame));
     real_call = (PyBytes_AS_STRING(pCode)[MyFrame_GetLasti(frame) + 1] == 0);
 #else
     // f_lasti is -1 for a true call, and a real byte offset for a generator re-entry.
@@ -543,7 +572,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
 #endif
 
     if (real_call) {
-        self->pcur_entry->last_line = -MyFrame_GetCode(frame)->co_firstlineno;
+        self->pcur_entry->last_line = -MyFrame_BorrowCode(frame)->co_firstlineno;
     }
     else {
         self->pcur_entry->last_line = PyFrame_GetLineNumber(frame);
@@ -630,7 +659,7 @@ CTracer_handle_line(CTracer *self, PyFrameObject *frame)
 
     STATS( self->stats.lines++; )
     if (self->pdata_stack->depth >= 0) {
-        SHOWLOG(PyFrame_GetLineNumber(frame), MyFrame_GetCode(frame)->co_filename, "line");
+        SHOWLOG(PyFrame_GetLineNumber(frame), MyFrame_BorrowCode(frame)->co_filename, "line");
         if (self->pcur_entry->file_data) {
             int lineno_from = -1;
             int lineno_to = -1;
@@ -703,12 +732,12 @@ CTracer_handle_return(CTracer *self, PyFrameObject *frame)
     if (CTracer_set_pdata_stack(self) < 0) {
         goto error;
     }
-    self->pcur_entry = &self->pdata_stack->stack[self->pdata_stack->depth];
 
     if (self->pdata_stack->depth >= 0) {
+        self->pcur_entry = &self->pdata_stack->stack[self->pdata_stack->depth];
         if (self->tracing_arcs && self->pcur_entry->file_data) {
             BOOL real_return = FALSE;
-            pCode = MyCode_GetCode(MyFrame_GetCode(frame));
+            pCode = MyCode_GetCode(MyFrame_BorrowCode(frame));
             int lasti = MyFrame_GetLasti(frame);
             Py_ssize_t code_size = PyBytes_GET_SIZE(pCode);
             unsigned char * code_bytes = (unsigned char *)PyBytes_AS_STRING(pCode);
@@ -717,7 +746,10 @@ CTracer_handle_return(CTracer *self, PyFrameObject *frame)
                 real_return = TRUE;
             }
             else {
-                real_return = (code_bytes[lasti + 2] != RESUME);
+#if ENV_LASTI_IS_YIELD
+                lasti += 2;
+#endif
+                real_return = (code_bytes[lasti] != RESUME);
             }
 #else
             /* Need to distinguish between RETURN_VALUE and YIELD_VALUE. Read
@@ -737,7 +769,7 @@ CTracer_handle_return(CTracer *self, PyFrameObject *frame)
             real_return = !(is_yield || is_yield_from);
 #endif
             if (real_return) {
-                int first = MyFrame_GetCode(frame)->co_firstlineno;
+                int first = MyFrame_BorrowCode(frame)->co_firstlineno;
                 if (CTracer_record_pair(self, self->pcur_entry->last_line, -first) < 0) {
                     goto error;
                 }
@@ -760,7 +792,7 @@ CTracer_handle_return(CTracer *self, PyFrameObject *frame)
         }
 
         /* Pop the stack. */
-        SHOWLOG(PyFrame_GetLineNumber(frame), MyFrame_GetCode(frame)->co_filename, "return");
+        SHOWLOG(PyFrame_GetLineNumber(frame), MyFrame_BorrowCode(frame)->co_filename, "return");
         self->pdata_stack->depth--;
         self->pcur_entry = &self->pdata_stack->stack[self->pdata_stack->depth];
     }
@@ -785,7 +817,7 @@ CTracer_trace(CTracer *self, PyFrameObject *frame, int what, PyObject *arg_unuse
     return RET_OK;
     #endif
 
-    if (!self->started) {
+    if (!atomic_load(&self->started)) {
         /* If CTracer.stop() has been called from another thread, the tracer
            is still active in the current thread. Let's deactivate ourselves
            now. */
@@ -802,20 +834,20 @@ CTracer_trace(CTracer *self, PyFrameObject *frame, int what, PyObject *arg_unuse
     if (what <= (int)(sizeof(what_sym)/sizeof(const char *))) {
         w = what_sym[what];
     }
-    ascii = PyUnicode_AsASCIIString(MyFrame_GetCode(frame)->co_filename);
+    ascii = PyUnicode_AsASCIIString(MyFrame_BorrowCode(frame)->co_filename);
     printf("%x trace: f:%x %s @ %s %d\n", (int)self, (int)frame, what_sym[what], PyBytes_AS_STRING(ascii), PyFrame_GetLineNumber(frame));
     Py_DECREF(ascii);
     #endif
 
     #if TRACE_LOG
-    ascii = PyUnicode_AsASCIIString(MyFrame_GetCode(frame)->co_filename);
+    ascii = PyUnicode_AsASCIIString(MyFrame_BorrowCode(frame)->co_filename);
     if (strstr(PyBytes_AS_STRING(ascii), start_file) && PyFrame_GetLineNumber(frame) == start_line) {
         logging = TRUE;
     }
     Py_DECREF(ascii);
     #endif
 
-    self->activity = TRUE;
+    atomic_store(&self->activity, TRUE);
 
     switch (what) {
     case PyTrace_CALL:
@@ -863,14 +895,6 @@ cleanup:
  * means it must be callable to be used in sys.settrace().
  *
  * So we make ourself callable, equivalent to invoking our trace function.
- *
- * To help with the process of replaying stored frames, this function has an
- * optional keyword argument:
- *
- *      def CTracer_call(frame, event, arg, lineno=0)
- *
- * If provided, the lineno argument is used as the line number, and the
- * frame's f_lineno member is ignored.
  */
 static PyObject *
 CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
@@ -878,9 +902,7 @@ CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
     PyFrameObject *frame;
     PyObject *what_str;
     PyObject *arg;
-    int lineno = 0;
     int what;
-    int orig_lineno;
     PyObject *ret = NULL;
     PyObject * ascii = NULL;
 
@@ -894,10 +916,10 @@ CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
         NULL
         };
 
-    static char *kwlist[] = {"frame", "event", "arg", "lineno", NULL};
+    static char *kwlist[] = {"frame", "event", "arg", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O!O|i:Tracer_call", kwlist,
-            &PyFrame_Type, &frame, &PyUnicode_Type, &what_str, &arg, &lineno)) {
+            &PyFrame_Type, &frame, &PyUnicode_Type, &what_str, &arg)) {
         goto done;
     }
 
@@ -914,26 +936,17 @@ CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
     }
 
     #if WHAT_LOG
-    ascii = PyUnicode_AsASCIIString(MyFrame_GetCode(frame)->co_filename);
+    ascii = PyUnicode_AsASCIIString(MyFrame_BorrowCode(frame)->co_filename);
     printf("pytrace: %s @ %s %d\n", what_sym[what], PyBytes_AS_STRING(ascii), PyFrame_GetLineNumber(frame));
     Py_DECREF(ascii);
     #endif
 
-    /* Save off the frame's lineno, and use the forced one, if provided. */
-    orig_lineno = frame->f_lineno;
-    if (lineno > 0) {
-        frame->f_lineno = lineno;
-    }
-
     /* Invoke the C function, and return ourselves. */
     if (CTracer_trace(self, frame, what, arg) == RET_OK) {
         Py_INCREF(self);
         ret = (PyObject *)self;
     }
 
-    /* Clean up. */
-    frame->f_lineno = orig_lineno;
-
     /* For better speed, install ourselves the C way so that future calls go
        directly to CTracer_trace, without this intermediate function.
 
@@ -967,9 +980,10 @@ done:
 static PyObject *
 CTracer_start(CTracer *self, PyObject *args_unused)
 {
+    assert(atomic_load(&self->started) == FALSE);
     PyEval_SetTrace((Py_tracefunc)CTracer_trace, (PyObject*)self);
-    self->started = TRUE;
     self->tracing_arcs = self->trace_arcs && PyObject_IsTrue(self->trace_arcs);
+    atomic_store(&self->started, TRUE);
 
     /* start() returns a trace function usable with sys.settrace() */
     Py_INCREF(self);
@@ -979,13 +993,11 @@ CTracer_start(CTracer *self, PyObject *args_unused)
 static PyObject *
 CTracer_stop(CTracer *self, PyObject *args_unused)
 {
-    if (self->started) {
-        /* Set the started flag only. The actual call to
-           PyEval_SetTrace(NULL, NULL) is delegated to the callback
-           itself to ensure that it called from the right thread.
-           */
-        self->started = FALSE;
-    }
+    /* Set the started flag only. The actual call to
+       PyEval_SetTrace(NULL, NULL) is delegated to the callback
+       itself to ensure that it called from the right thread.
+       */
+    atomic_store(&self->started, FALSE);
 
     Py_RETURN_NONE;
 }
@@ -993,7 +1005,7 @@ CTracer_stop(CTracer *self, PyObject *args_unused)
 static PyObject *
 CTracer_activity(CTracer *self, PyObject *args_unused)
 {
-    if (self->activity) {
+    if (atomic_load(&self->activity)) {
         Py_RETURN_TRUE;
     }
     else {
@@ -1004,7 +1016,7 @@ CTracer_activity(CTracer *self, PyObject *args_unused)
 static PyObject *
 CTracer_reset_activity(CTracer *self, PyObject *args_unused)
 {
-    self->activity = FALSE;
+    atomic_store(&self->activity, FALSE);
     Py_RETURN_NONE;
 }
 
@@ -1013,7 +1025,7 @@ CTracer_get_stats(CTracer *self, PyObject *args_unused)
 {
 #if COLLECT_STATS
     return Py_BuildValue(
-        "{sI,sI,sI,sI,sI,sI,sI,sI,si,sI,sI,sI}",
+        "{sI,sI,sI,sI,sI,sI,si,sI,sI,sI}",
         "calls", self->stats.calls,
         "lines", self->stats.lines,
         "returns", self->stats.returns,
@@ -1062,6 +1074,12 @@ CTracer_members[] = {
     { "switch_context",     T_OBJECT, offsetof(CTracer, switch_context), 0,
             PyDoc_STR("Function for switching to a new context.") },
 
+    { "lock_data",          T_OBJECT, offsetof(CTracer, lock_data), 0,
+            PyDoc_STR("Function for locking access to self.data.") },
+
+    { "unlock_data",        T_OBJECT, offsetof(CTracer, unlock_data), 0,
+            PyDoc_STR("Function for unlocking access to self.data.") },
+
     { "disable_plugin",     T_OBJECT, offsetof(CTracer, disable_plugin), 0,
             PyDoc_STR("Function for disabling a plugin.") },
 
