@@ -14,7 +14,8 @@ import os
 import os.path
 import sys
 
-from typing import Any, Callable, Iterable, Iterator, Optional, Tuple, cast
+from collections.abc import Iterable
+from typing import Any, Callable, cast
 
 import pytest
 
@@ -63,7 +64,7 @@ class TempDirMixin:
     run_in_temp_dir = True
 
     @pytest.fixture(autouse=True)
-    def _temp_dir(self, tmp_path_factory: pytest.TempPathFactory) -> Iterator[None]:
+    def _temp_dir(self, tmp_path_factory: pytest.TempPathFactory) -> Iterable[None]:
         """Create a temp dir for the tests, if they want it."""
         if self.run_in_temp_dir:
             tmpdir = tmp_path_factory.mktemp("t")
@@ -83,7 +84,7 @@ class TempDirMixin:
         filename: str,
         text: str = "",
         bytes: bytes = b"",
-        newline: Optional[str] = None,
+        newline: str | None = None,
     ) -> str:
         """Make a file. See `tests.helpers.make_file`"""
         # pylint: disable=redefined-builtin     # bytes
@@ -131,14 +132,15 @@ class StdStreamCapturingMixin:
     invocation will only return the delta.
 
     """
+
     @pytest.fixture(autouse=True)
     def _capcapsys(self, capsys: pytest.CaptureFixture[str]) -> None:
         """Grab the fixture so our methods can use it."""
         self.capsys = capsys
 
-    def stdouterr(self) -> Tuple[str, str]:
+    def stdouterr(self) -> tuple[str, str]:
         """Returns (out, err), two strings for stdout and stderr."""
-        return cast(Tuple[str, str], self.capsys.readouterr())
+        return cast(tuple[str, str], self.capsys.readouterr())
 
     def stdout(self) -> str:
         """Returns a string, the captured stdout."""
